@@ -1,6 +1,6 @@
 /* sexp.c  -  S-Expression handling
  * Copyright (C) 1999, 2000, 2001, 2002, 2003,
- *               2004, 2006, 2007, 2008  Free Software Foundation, Inc.
+ *               2004, 2006, 2007, 2008, 2011  Free Software Foundation, Inc.
  *
  * This file is part of Libgcrypt.
  *
@@ -30,7 +30,6 @@
 
 #define GCRYPT_NO_MPI_MACROS 1
 #include "g10lib.h"
-#include "memory.h"
 
 typedef struct gcry_sexp *NODE;
 typedef unsigned short DATALEN;
@@ -54,17 +53,22 @@ struct gcry_sexp
 
 #define TOKEN_SPECIALS  "-./_:*+="
 
+static gcry_error_t
+vsexp_sscan (gcry_sexp_t *retsexp, size_t *erroff,
+	     const char *buffer, size_t length, int argflag,
+	     void **arg_list, va_list arg_ptr);
+
 static gcry_error_t
 sexp_sscan (gcry_sexp_t *retsexp, size_t *erroff,
 	    const char *buffer, size_t length, int argflag,
-	    va_list arg_ptr, void **arg_list);
+	    void **arg_list, ...);
 
 /* Return true if P points to a byte containing a whitespace according
    to the S-expressions definition. */
 #undef whitespacep
 static GPG_ERR_INLINE int
 whitespacep (const char *p)
-{ 
+{
   switch (*p)
     {
     case ' ': case '\t': case '\v': case '\f': case '\r': case '\n': return 1;
@@ -94,7 +98,7 @@ dump_string (const byte *p, size_t n, int delim )
 {
   for (; n; n--, p++ )
     {
-      if ((*p & 0x80) || iscntrl( *p ) || *p == delim ) 
+      if ((*p & 0x80) || iscntrl( *p ) || *p == delim )
         {
           if( *p == '\n' )
             log_printf ("\\n");
@@ -106,7 +110,7 @@ dump_string (const byte *p, size_t n, int delim )
             log_printf ("\\v");
 	    else if( *p == '\b' )
               log_printf ("\\b");
-          else if( !*p )       
+          else if( !*p )
             log_printf ("\\0");
           else
             log_printf ("\\x%02x", *p );
@@ -174,7 +178,7 @@ normalize ( gcry_sexp_t list )
   if ( !list )
     return NULL;
   p = list->d;
-  if ( *p == ST_STOP ) 
+  if ( *p == ST_STOP )
     {
       /* this is "" */
       gcry_sexp_release ( list );
@@ -186,7 +190,7 @@ normalize ( gcry_sexp_t list )
       gcry_sexp_release ( list );
       return NULL;
     }
-  
+
   return list;
 }
 
@@ -196,13 +200,13 @@ normalize ( gcry_sexp_t list )
    the buffer is transferred to the newly created object.  FREEFNC
    should be the freefnc used to release BUFFER; there is no guarantee
    at which point this function is called; most likey you want to use
-   free() or gcry_free(). 
- 
+   free() or gcry_free().
+
    Passing LENGTH and AUTODETECT as 0 is allowed to indicate that
    BUFFER points to a valid canonical encoded S-expression.  A LENGTH
    of 0 and AUTODETECT 1 indicates that buffer points to a
    null-terminated string.
-  
+
    This function returns 0 and and the pointer to the new object in
    RETSEXP or an error code in which case RETSEXP is set to NULL.  */
 gcry_error_t
@@ -211,7 +215,6 @@ gcry_sexp_create (gcry_sexp_t *retsexp, void *buffer, size_t length,
 {
   gcry_error_t errcode;
   gcry_sexp_t se;
-  volatile va_list dummy_arg_ptr;
 
   if (!retsexp)
     return gcry_error (GPG_ERR_INV_ARG);
@@ -231,7 +234,7 @@ gcry_sexp_create (gcry_sexp_t *retsexp, void *buffer, size_t length,
       length = strlen ((char *)buffer);
     }
 
-  errcode = sexp_sscan (&se, NULL, buffer, length, 0, dummy_arg_ptr, NULL);
+  errcode = sexp_sscan (&se, NULL, buffer, length, 0, NULL);
   if (errcode)
     return errcode;
 
@@ -280,7 +283,7 @@ gcry_sexp_release( gcry_sexp_t sexp )
                   break;
                 case ST_CLOSE:
                   break;
-                case ST_DATA: 
+                case ST_DATA:
                   {
                     DATALEN n;
                     memcpy ( &n, p, sizeof n );
@@ -383,7 +386,7 @@ gcry_sexp_find_token( const gcry_sexp_t list, const char *tok, size_t toklen )
 {
   const byte *p;
   DATALEN n;
-  
+
   if ( !list )
     return NULL;
 
@@ -393,7 +396,7 @@ gcry_sexp_find_token( const gcry_sexp_t list, const char *tok, size_t toklen )
   p = list->d;
   while ( *p != ST_STOP )
     {
-      if ( *p == ST_OPEN && p[1] == ST_DATA ) 
+      if ( *p == ST_OPEN && p[1] == ST_DATA )
         {
           const byte *head = p;
 
@@ -407,7 +410,7 @@ gcry_sexp_find_token( const gcry_sexp_t list, const char *tok, size_t toklen )
               int level = 1;
 
               /* Look for the end of the list.  */
-              for ( p += n; level; p++ ) 
+              for ( p += n; level; p++ )
                 {
                   if ( *p == ST_DATA )
                     {
@@ -415,15 +418,15 @@ gcry_sexp_find_token( const gcry_sexp_t list, const char *tok, size_t toklen )
 			p += sizeof n + n;
 			p--; /* Compensate for later increment. */
 		    }
-                  else if ( *p == ST_OPEN ) 
+                  else if ( *p == ST_OPEN )
                     {
                       level++;
 		    }
-                  else if ( *p == ST_CLOSE ) 
+                  else if ( *p == ST_CLOSE )
                     {
                       level--;
 		    }
-                  else if ( *p == ST_STOP ) 
+                  else if ( *p == ST_STOP )
                     {
                       BUG ();
 		    }
@@ -502,15 +505,15 @@ get_internal_buffer (const gcry_sexp_t list, size_t *r_off)
   DATALEN n;
   int type;
   int level = 0;
-  
+
   *r_off = 0;
   if (list)
     {
       p = list->d;
-      while ( (type=*p) != ST_STOP ) 
+      while ( (type=*p) != ST_STOP )
         {
           p++;
-          if (type == ST_DATA) 
+          if (type == ST_DATA)
             {
               memcpy (&n, p, sizeof n);
               p += sizeof n + n;
@@ -573,13 +576,25 @@ gcry_sexp_nth( const gcry_sexp_t list, int number )
     p++;
 
     if ( *p == ST_DATA ) {
-	memcpy ( &n, p, sizeof n ); p += sizeof n;
-	newlist = gcry_malloc ( sizeof *newlist + n + 1 );
+        memcpy ( &n, p, sizeof n );
+        /* Allocate 1 (=sizeof *newlist) byte for ST_OPEN
+                    1 byte for ST_DATA
+                    sizeof n byte for n
+                    n byte for the data
+                    1 byte for ST_CLOSE
+                    1 byte for ST_STOP */
+        newlist = gcry_malloc ( sizeof *newlist + 1 + sizeof n + n + 2 );
         if (!newlist)
-          return NULL;
-	d = newlist->d;
-	memcpy ( d, p, n ); d += n;
-	*d++ = ST_STOP;
+            return NULL;
+        d = newlist->d;
+        *d = ST_OPEN;   /* Put the ST_OPEN flag */
+        d++;            /* Move forward */
+        /* Copy ST_DATA, n and the data from p to d */
+        memcpy ( d, p, 1 + sizeof n + n );
+        d += 1 + sizeof n + n;  /* Move after the data copied */
+        *d = ST_CLOSE;          /* Put the ST_CLOSE flag */
+        d++;                    /* Move forward */
+        *d = ST_STOP;           /* Put the ST_STOP flag */
     }
     else if ( *p == ST_OPEN ) {
 	const byte *head = p;
@@ -632,9 +647,9 @@ sexp_nth_data (const gcry_sexp_t list, int number, size_t *datalen)
   const byte *p;
   DATALEN n;
   int level = 0;
-  
+
   *datalen = 0;
-  if ( !list ) 
+  if ( !list )
     return NULL;
 
   p = list->d;
@@ -644,9 +659,9 @@ sexp_nth_data (const gcry_sexp_t list, int number, size_t *datalen)
     return NULL;     /* Not a list but N > 0 requested. */
 
   /* Skip over N elements. */
-  while ( number > 0 ) 
+  while ( number > 0 )
     {
-      if ( *p == ST_DATA ) 
+      if ( *p == ST_DATA )
         {
           memcpy ( &n, ++p, sizeof n );
           p += sizeof n + n;
@@ -654,17 +669,17 @@ sexp_nth_data (const gcry_sexp_t list, int number, size_t *datalen)
           if ( !level )
             number--;
 	}
-      else if ( *p == ST_OPEN ) 
+      else if ( *p == ST_OPEN )
         {
           level++;
 	}
-      else if ( *p == ST_CLOSE ) 
+      else if ( *p == ST_CLOSE )
         {
           level--;
           if ( !level )
             number--;
 	}
-      else if ( *p == ST_STOP ) 
+      else if ( *p == ST_STOP )
         {
           return NULL;
 	}
@@ -678,7 +693,7 @@ sexp_nth_data (const gcry_sexp_t list, int number, size_t *datalen)
       *datalen = n;
       return (const char*)p + sizeof n;
     }
-  
+
   return NULL;
 }
 
@@ -856,13 +871,13 @@ static gpg_err_code_t
 make_space ( struct make_space_ctx *c, size_t n )
 {
   size_t used = c->pos - c->sexp->d;
-  
+
   if ( used + n + sizeof(DATALEN) + 1 >= c->allocated )
     {
       gcry_sexp_t newsexp;
       byte *newhead;
       size_t newsize;
-      
+
       newsize = c->allocated + 2*(n+sizeof(DATALEN)+1);
       if (newsize <= c->allocated)
         return GPG_ERR_TOO_LARGE;
@@ -910,9 +925,8 @@ unquote_string (const char *string, size_t length, unsigned char *buf)
                 {
                   s++; n--;
                 }
-              esc = 0;
               break;
-              
+
             case '\n':  /* ignore LF[,CR] */
               if (n>1 && s[1] == '\r')
                 {
@@ -944,7 +958,7 @@ unquote_string (const char *string, size_t length, unsigned char *buf)
         esc = 1;
       else
         *d++ = *s;
-    } 
+    }
 
   return d - buf;
 }
@@ -952,17 +966,17 @@ unquote_string (const char *string, size_t length, unsigned char *buf)
 /****************
  * Scan the provided buffer and return the S expression in our internal
  * format.  Returns a newly allocated expression.  If erroff is not NULL and
- * a parsing error has occured, the offset into buffer will be returned.
+ * a parsing error has occurred, the offset into buffer will be returned.
  * If ARGFLAG is true, the function supports some printf like
  * expressions.
  *  These are:
  *	%m - MPI
  *	%s - string (no autoswitch to secure allocation)
  *	%d - integer stored as string (no autoswitch to secure allocation)
- *      %b - memory buffer; this takes _two_ arguments: an integer with the 
+ *      %b - memory buffer; this takes _two_ arguments: an integer with the
  *           length of the buffer and a pointer to the buffer.
  *      %S - Copy an gcry_sexp_t here.  The S-expression needs to be a
- *           regular one, starting with a parenthesis. 
+ *           regular one, starting with a parenthesis.
  *           (no autoswitch to secure allocation)
  *  all other format elements are currently not defined and return an error.
  *  this includes the "%%" sequence becauce the percent sign is not an
@@ -975,9 +989,9 @@ unquote_string (const char *string, size_t length, unsigned char *buf)
  * regardless whether it is needed or not.
  */
 static gcry_error_t
-sexp_sscan (gcry_sexp_t *retsexp, size_t *erroff,
-	    const char *buffer, size_t length, int argflag,
-	    va_list arg_ptr, void **arg_list)
+vsexp_sscan (gcry_sexp_t *retsexp, size_t *erroff,
+	     const char *buffer, size_t length, int argflag,
+	     void **arg_list, va_list arg_ptr)
 {
   gcry_err_code_t err = 0;
   static const char tokenchars[] =
@@ -1004,7 +1018,7 @@ sexp_sscan (gcry_sexp_t *retsexp, size_t *erroff,
   if (!erroff)
     erroff = &dummy_erroff;
 
-  /* Depending on wether ARG_LIST is non-zero or not, this macro gives
+  /* Depending on whether ARG_LIST is non-zero or not, this macro gives
      us the next argument, either from the variable argument list as
      specified by ARG_PTR or from the argument array ARG_LIST.  */
 #define ARG_NEXT(storage, type)                          \
@@ -1098,7 +1112,7 @@ sexp_sscan (gcry_sexp_t *retsexp, size_t *erroff,
 		  n -= 2;
 		  quoted_esc = 0;
 		  break;
-		  
+
 		case 'x':
 		  if (!((n > 2) && hexdigitp (p+1) && hexdigitp (p+2)))
 		    {
@@ -1147,7 +1161,7 @@ sexp_sscan (gcry_sexp_t *retsexp, size_t *erroff,
 		 never be larger. */
 	      unsigned char *save;
 	      size_t len;
-	      
+
 	      quoted++; /* Skip leading quote.  */
 	      MAKE_SPACE (p - quoted);
 	      *c.pos++ = ST_DATA;
@@ -1247,44 +1261,85 @@ sexp_sscan (gcry_sexp_t *retsexp, size_t *erroff,
 	}
       else if (percent)
 	{
-	  if (*p == 'm')
+	  if (*p == 'm' || *p == 'M')
 	    {
 	      /* Insert an MPI.  */
 	      gcry_mpi_t m;
 	      size_t nm = 0;
+              int mpifmt = *p == 'm'? GCRYMPI_FMT_STD: GCRYMPI_FMT_USG;
 
 	      ARG_NEXT (m, gcry_mpi_t);
-	      
-	      if (gcry_mpi_print (GCRYMPI_FMT_STD, NULL, 0, &nm, m))
-		BUG ();
 
-	      MAKE_SPACE (nm);
-	      if (!gcry_is_secure (c.sexp->d)
-		  && gcry_mpi_get_flag ( m, GCRYMPI_FLAG_SECURE))
-		{
-		  /* We have to switch to secure allocation.  */
-		  gcry_sexp_t newsexp;
-		  byte *newhead;
+              if (gcry_mpi_get_flag (m, GCRYMPI_FLAG_OPAQUE))
+                {
+                  void *mp;
+                  unsigned int nbits;
 
-		  newsexp = gcry_malloc_secure (sizeof *newsexp
-                                                + c.allocated - 1);
-                  if (!newsexp)
+                  mp = gcry_mpi_get_opaque (m, &nbits);
+                  nm = (nbits+7)/8;
+                  if (mp && nm)
                     {
-                      err = gpg_err_code_from_errno (errno);
-                      goto leave;
-                    }
-		  newhead = newsexp->d;
-		  memcpy (newhead, c.sexp->d, (c.pos - c.sexp->d));
-		  c.pos = newhead + (c.pos - c.sexp->d);
-		  gcry_free (c.sexp);
-		  c.sexp = newsexp;
-		}
+                      MAKE_SPACE (nm);
+                      if (!gcry_is_secure (c.sexp->d)
+                          && gcry_mpi_get_flag (m, GCRYMPI_FLAG_SECURE))
+                        {
+                          /* We have to switch to secure allocation.  */
+                          gcry_sexp_t newsexp;
+                          byte *newhead;
 
-	      *c.pos++ = ST_DATA;
-	      STORE_LEN (c.pos, nm);
-	      if (gcry_mpi_print (GCRYMPI_FMT_STD, c.pos, nm, &nm, m))
-		BUG ();
-	      c.pos += nm;
+                          newsexp = gcry_malloc_secure (sizeof *newsexp
+                                                        + c.allocated - 1);
+                          if (!newsexp)
+                            {
+                              err = gpg_err_code_from_errno (errno);
+                              goto leave;
+                            }
+                          newhead = newsexp->d;
+                          memcpy (newhead, c.sexp->d, (c.pos - c.sexp->d));
+                          c.pos = newhead + (c.pos - c.sexp->d);
+                          gcry_free (c.sexp);
+                          c.sexp = newsexp;
+                        }
+
+                      *c.pos++ = ST_DATA;
+                      STORE_LEN (c.pos, nm);
+                      memcpy (c.pos, mp, nm);
+                      c.pos += nm;
+                    }
+                }
+              else
+                {
+                  if (gcry_mpi_print (mpifmt, NULL, 0, &nm, m))
+                    BUG ();
+
+                  MAKE_SPACE (nm);
+                  if (!gcry_is_secure (c.sexp->d)
+                      && gcry_mpi_get_flag ( m, GCRYMPI_FLAG_SECURE))
+                    {
+                      /* We have to switch to secure allocation.  */
+                      gcry_sexp_t newsexp;
+                      byte *newhead;
+
+                      newsexp = gcry_malloc_secure (sizeof *newsexp
+                                                    + c.allocated - 1);
+                      if (!newsexp)
+                        {
+                          err = gpg_err_code_from_errno (errno);
+                          goto leave;
+                        }
+                      newhead = newsexp->d;
+                      memcpy (newhead, c.sexp->d, (c.pos - c.sexp->d));
+                      c.pos = newhead + (c.pos - c.sexp->d);
+                      gcry_free (c.sexp);
+                      c.sexp = newsexp;
+                    }
+
+                  *c.pos++ = ST_DATA;
+                  STORE_LEN (c.pos, nm);
+                  if (gcry_mpi_print (mpifmt, c.pos, nm, &nm, m))
+                    BUG ();
+                  c.pos += nm;
+                }
 	    }
 	  else if (*p == 's')
 	    {
@@ -1294,7 +1349,7 @@ sexp_sscan (gcry_sexp_t *retsexp, size_t *erroff,
 
 	      ARG_NEXT (astr, const char *);
 	      alen = strlen (astr);
-	      
+
 	      MAKE_SPACE (alen);
 	      *c.pos++ = ST_DATA;
 	      STORE_LEN (c.pos, alen);
@@ -1309,7 +1364,7 @@ sexp_sscan (gcry_sexp_t *retsexp, size_t *erroff,
 
 	      ARG_NEXT (alen, int);
 	      ARG_NEXT (astr, const char *);
-	      
+
 	      MAKE_SPACE (alen);
 	      if (alen
                   && !gcry_is_secure (c.sexp->d)
@@ -1343,8 +1398,8 @@ sexp_sscan (gcry_sexp_t *retsexp, size_t *erroff,
 	      /* Insert an integer as string.  */
 	      int aint;
 	      size_t alen;
-	      char buf[20];
-	      
+	      char buf[35];
+
 	      ARG_NEXT (aint, int);
 	      sprintf (buf, "%d", aint);
 	      alen = strlen (buf);
@@ -1354,6 +1409,22 @@ sexp_sscan (gcry_sexp_t *retsexp, size_t *erroff,
 	      memcpy (c.pos, buf, alen);
 	      c.pos += alen;
 	    }
+	  else if (*p == 'u')
+	    {
+	      /* Insert an unsigned integer as string.  */
+	      unsigned int aint;
+	      size_t alen;
+	      char buf[35];
+
+	      ARG_NEXT (aint, unsigned int);
+	      sprintf (buf, "%u", aint);
+	      alen = strlen (buf);
+	      MAKE_SPACE (alen);
+	      *c.pos++ = ST_DATA;
+	      STORE_LEN (c.pos, alen);
+	      memcpy (c.pos, buf, alen);
+	      c.pos += alen;
+	    }
 	  else if (*p == 'S')
 	    {
 	      /* Insert a gcry_sexp_t.  */
@@ -1509,61 +1580,63 @@ sexp_sscan (gcry_sexp_t *retsexp, size_t *erroff,
 #undef STORE_LEN
 }
 
-gcry_error_t
-gcry_sexp_build (gcry_sexp_t *retsexp, size_t *erroff, const char *format, ...)
+
+static gcry_error_t
+sexp_sscan (gcry_sexp_t *retsexp, size_t *erroff,
+	    const char *buffer, size_t length, int argflag,
+	    void **arg_list, ...)
 {
   gcry_error_t rc;
   va_list arg_ptr;
-  
-  va_start (arg_ptr, format);
-  rc = sexp_sscan (retsexp, erroff, format, strlen(format), 1,
-		   arg_ptr, NULL);
+
+  va_start (arg_ptr, arg_list);
+  rc = vsexp_sscan (retsexp, erroff, buffer, length, argflag,
+		    arg_list, arg_ptr);
   va_end (arg_ptr);
-  
+
   return rc;
 }
 
 
 gcry_error_t
-_gcry_sexp_vbuild (gcry_sexp_t *retsexp, size_t *erroff, 
+gcry_sexp_build (gcry_sexp_t *retsexp, size_t *erroff, const char *format, ...)
+{
+  gcry_error_t rc;
+  va_list arg_ptr;
+
+  va_start (arg_ptr, format);
+  rc = vsexp_sscan (retsexp, erroff, format, strlen(format), 1,
+		    NULL, arg_ptr);
+  va_end (arg_ptr);
+
+  return rc;
+}
+
+
+gcry_error_t
+_gcry_sexp_vbuild (gcry_sexp_t *retsexp, size_t *erroff,
                    const char *format, va_list arg_ptr)
 {
-  return sexp_sscan (retsexp, erroff, format, strlen(format), 1,
-                     arg_ptr, NULL);
+  return vsexp_sscan (retsexp, erroff, format, strlen(format), 1,
+		      NULL, arg_ptr);
 }
 
+
 /* Like gcry_sexp_build, but uses an array instead of variable
    function arguments.  */
 gcry_error_t
 gcry_sexp_build_array (gcry_sexp_t *retsexp, size_t *erroff,
 		       const char *format, void **arg_list)
 {
-  /* We don't need the va_list because it is controlled by the
-     following flag, however we have to pass it but can't initialize
-     it as there is no portable way to do so.  volatile is needed to
-     suppress the compiler warning */
-  volatile va_list dummy_arg_ptr;
-  
-  gcry_error_t rc;
-
-  rc = sexp_sscan (retsexp, erroff, format, strlen(format), 1,
-		   dummy_arg_ptr, arg_list);
-
-  return rc;
+  return sexp_sscan (retsexp, erroff, format, strlen(format), 1, arg_list);
 }
 
+
 gcry_error_t
 gcry_sexp_sscan (gcry_sexp_t *retsexp, size_t *erroff,
 		 const char *buffer, size_t length)
 {
-  /* We don't need the va_list because it is controlled by the
-     following flag, however we have to pass it but can't initialize
-     it as there is no portable way to do so.  volatile is needed to
-     suppress the compiler warning */
-  volatile va_list dummy_arg_ptr;
-
-  return sexp_sscan (retsexp, erroff, buffer, length, 0,
-		     dummy_arg_ptr, NULL);
+  return sexp_sscan (retsexp, erroff, buffer, length, 0, NULL);
 }
 
 
@@ -1580,7 +1653,7 @@ suitable_encoding (const unsigned char *buffer, size_t length)
 
   if (!length)
     return 1;
-  
+
   for (s=buffer; length; s++, length--)
     {
       if ( (*s < 0x20 || (*s >= 0x7f && *s <= 0xa0))
@@ -1632,10 +1705,10 @@ convert_to_string (const unsigned char *s, size_t len, char *dest)
             case '\"': *p++ = '\\'; *p++ = '\"';  break;
             case '\'': *p++ = '\\'; *p++ = '\'';  break;
             case '\\': *p++ = '\\'; *p++ = '\\';  break;
-            default: 
+            default:
               if ( (*s < 0x20 || (*s >= 0x7f && *s <= 0xa0)))
                 {
-                  sprintf (p, "\\x%02x", *s); 
+                  sprintf (p, "\\x%02x", *s);
                   p += 4;
                 }
               else
@@ -1652,16 +1725,16 @@ convert_to_string (const unsigned char *s, size_t len, char *dest)
         {
           switch (*s)
             {
-            case '\b': 
-            case '\t': 
-            case '\v': 
-            case '\n': 
-            case '\f': 
-            case '\r': 
+            case '\b':
+            case '\t':
+            case '\v':
+            case '\n':
+            case '\f':
+            case '\r':
             case '\"':
             case '\'':
             case '\\': count += 2; break;
-            default: 
+            default:
               if ( (*s < 0x20 || (*s >= 0x7f && *s <= 0xa0)))
                 count += 4;
               else
@@ -1700,7 +1773,7 @@ gcry_sexp_sprint (const gcry_sexp_t list, int mode,
   char numbuf[20];
   size_t len = 0;
   int i, indent = 0;
-  
+
   s = list? list->d : empty;
   d = buffer;
   while ( *s != ST_STOP )
@@ -1712,18 +1785,18 @@ gcry_sexp_sprint (const gcry_sexp_t list, int mode,
           if ( mode != GCRYSEXP_FMT_CANON )
             {
               if (indent)
-                len++; 
+                len++;
               len += indent;
             }
           len++;
-          if ( buffer ) 
+          if ( buffer )
             {
               if ( len >= maxlength )
                 return 0;
               if ( mode != GCRYSEXP_FMT_CANON )
                 {
                   if (indent)
-                    *d++ = '\n'; 
+                    *d++ = '\n';
                   for (i=0; i < indent; i++)
                     *d++ = ' ';
                 }
@@ -1734,7 +1807,7 @@ gcry_sexp_sprint (const gcry_sexp_t list, int mode,
         case ST_CLOSE:
           s++;
           len++;
-          if ( buffer ) 
+          if ( buffer )
             {
               if ( len >= maxlength )
                 return 0;
@@ -1797,7 +1870,7 @@ gcry_sexp_sprint (const gcry_sexp_t list, int mode,
             {
               sprintf (numbuf, "%u:", (unsigned int)n );
               len += strlen (numbuf) + n;
-              if ( buffer ) 
+              if ( buffer )
                 {
                   if ( len >= maxlength )
 		    return 0;
@@ -1818,10 +1891,10 @@ gcry_sexp_sprint (const gcry_sexp_t list, int mode,
         {
           if ( len >= maxlength )
             return 0;
-          *d++ = '\n'; 
+          *d++ = '\n';
         }
     }
-  if (buffer) 
+  if (buffer)
     {
       if ( len >= maxlength )
         return 0;
@@ -1834,14 +1907,14 @@ gcry_sexp_sprint (const gcry_sexp_t list, int mode,
 }
 
 
-/* Scan a cannocial encoded buffer with implicit length values and
+/* Scan a canonical encoded buffer with implicit length values and
    return the actual length this S-expression uses.  For a valid S-Exp
    it should never return 0.  If LENGTH is not zero, the maximum
    length to scan is given - this can be used for syntax checks of
    data passed from outside. errorcode and erroff may both be passed as
    NULL.  */
 size_t
-gcry_sexp_canon_len (const unsigned char *buffer, size_t length, 
+gcry_sexp_canon_len (const unsigned char *buffer, size_t length,
                      size_t *erroff, gcry_error_t *errcode)
 {
   const unsigned char *p;
@@ -1875,7 +1948,7 @@ gcry_sexp_canon_len (const unsigned char *buffer, size_t length,
           *errcode = gcry_error (GPG_ERR_SEXP_STRING_TOO_LONG);
           return 0;
         }
-      
+
       if (datalen)
         {
           if (*p == ':')
@@ -1892,7 +1965,7 @@ gcry_sexp_canon_len (const unsigned char *buffer, size_t length,
 	    }
           else if (digitp(p))
             datalen = datalen*10 + atoi_1(p);
-          else 
+          else
             {
               *erroff = count;
               *errcode = gcry_error (GPG_ERR_SEXP_INV_LEN_SPEC);
@@ -1928,7 +2001,7 @@ gcry_sexp_canon_len (const unsigned char *buffer, size_t length,
 	}
       else if (*p == '[')
         {
-          if (disphint) 
+          if (disphint)
             {
               *erroff = count;
               *errcode = gcry_error (GPG_ERR_SEXP_NESTED_DH);
@@ -1938,7 +2011,7 @@ gcry_sexp_canon_len (const unsigned char *buffer, size_t length,
 	}
       else if (*p == ']')
         {
-          if ( !disphint ) 
+          if ( !disphint )
             {
               *erroff = count;
               *errcode = gcry_error (GPG_ERR_SEXP_UNMATCHED_DH);
@@ -1949,7 +2022,7 @@ gcry_sexp_canon_len (const unsigned char *buffer, size_t length,
       else if (digitp (p) )
         {
           if (*p == '0')
-            { 
+            {
               *erroff = count;
               *errcode = gcry_error (GPG_ERR_SEXP_ZERO_PREFIX);
               return 0;
@@ -1963,7 +2036,7 @@ gcry_sexp_canon_len (const unsigned char *buffer, size_t length,
           return 0;
 	}
       else
-        { 
+        {
           *erroff = count;
           *errcode = gcry_error (GPG_ERR_SEXP_BAD_CHARACTER);
           return 0;
